use serenity::{prelude::GatewayIntents, Client};

use crate::environment::get_env_variable;

use super::event_handler::Handler;

pub struct DiscordClient {
    serenity_client: Option<Client>,
}

impl DiscordClient {
    pub async fn new() -> Self {
        let discord_token = get_env_variable("DISCORD_TOKEN");
        let intents = GatewayIntents::GUILD_MESSAGES | GatewayIntents::MESSAGE_CONTENT;

        let serenity_client = match Client::builder(&discord_token, intents)
            .event_handler(Handler)
            .await
        {
            Ok(c) => c,
            Err(e) => panic!("Failed to instantiate Discord Client: {e}"),
        };

        DiscordClient {
            serenity_client: Some(serenity_client),
        }
    }

    pub async fn start(&mut self) {
        if let Err(e) = self
            .serenity_client
            .as_mut()
            .expect("No bot instance was created.")
            .start()
            .await
        {
            panic!("Could not connect the bot: {e}");
        }
    }
}
